/*============================================================================
 *  Dfinitions des fonctions de base
 *   ralisant les sorties pour l'interfacage avec le Noyau
 *============================================================================*/

/*
                      Code_Saturne version 1.3
                      ------------------------

  This file is part of the Code_Saturne Preprocessor, element of the
  Code_Saturne CFD tool.

  Copyright (C) 1999-2007 EDF S.A., France

  contact: saturne-support@edf.fr

  The Code_Saturne Preprocessor is free software; you can redistribute it
  and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  The Code_Saturne Preprocessor is distributed in the hope that it will be
  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with the Code_Saturne Preprocessor; if not, write to the
  Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor,
  Boston, MA  02110-1301  USA
*/


/*============================================================================
 *                                 Visibilit
 *============================================================================*/

#include "ecs_config.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' librairie standard C ou BFT
 *----------------------------------------------------------------------------*/

#include <assert.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <bft_mem.h>
#include <bft_file.h>
#include <bft_error.h>
#include <bft_printf.h>


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles du  paquetage global "Utilitaire"
 *----------------------------------------------------------------------------*/

#include "ecs_chaine_glob.h"
#include "ecs_def.h"


/*----------------------------------------------------------------------------
 *  Fichier  `include' du  paquetage courant associ au fichier courant
 *----------------------------------------------------------------------------*/

#include "ecs_comm.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' prives du  paquetage courant
 *----------------------------------------------------------------------------*/

#include "ecs_comm_priv.h"


/*============================================================================
 *  Constantes et Macros
 *============================================================================*/

#define ECS_LOC_COMM_REEL_4 4
#define ECS_LOC_COMM_REEL_8 8


/*============================================================================
 *  Variables globales statiques
 *============================================================================*/


/*============================================================================
 *                       Prototypes de fonctions prives
 *============================================================================*/


/*----------------------------------------------------------------------------
 *  Fonction qui construit le descripteur du fichier d'interface et initialise
 *   ce fichier par l'envoi ou la lecture d'une ventuelle "chane magique"
 *   servant a vrifier le bon format des fichiers
 *----------------------------------------------------------------------------*/

static void ecs_loc_comm__fic_ouvre
(
       ecs_comm_t        *comm         ,
 const char       *const  nom_fic_comm
) ;


/*----------------------------------------------------------------------------
 *  Affichage de l'attente d'echange  d'un message
 *----------------------------------------------------------------------------*/

static void ecs_loc_comm__echo_pre
(
 const ecs_comm_t      *const comm
) ;


/*----------------------------------------------------------------------------
 *  Affichage de l'entete d'une rubrique
 *
 *  1. numro  de la rubrique
 *  2. nom     de la rubrique
 *  3. nombre          d'lments
 *  4. nom     du type d'lments
 *----------------------------------------------------------------------------*/

static void ecs_loc_comm__echo_entete
(
       ecs_int_t              num_rub     ,
 const char            *const nom_rub     ,
       ecs_int_t              nbr_elt_rub ,
 const char            *const nom_typ_e
) ;


/*----------------------------------------------------------------------------
 *  Affichage (partiel) du contenu d'une rubrique
 *----------------------------------------------------------------------------*/

static void ecs_loc_comm__echo_donnees
(
 const ecs_comm_t    *const comm,
       ecs_int_t            nbr_elt_rub,
       ecs_type_t     const typ_elt_e,
       void          *const elt_rub
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui crit un enregistrement dans le fichier d'interface
 *----------------------------------------------------------------------------*/

static void ecs_loc_comm__ecrit_rec
(
 const ecs_comm_t  *const comm  ,
 const ecs_byte_t  *      rec   ,
 const size_t             nbr   ,
       ecs_type_t         typ_e
) ;


/*----------------------------------------------------------------------------
 *  Fonction d'impression d'un tableau en ASCII
 *
 *  Si les valeurs sont de type "caractre", les valeurs sont crites
 *   sur une seule ligne
 *----------------------------------------------------------------------------*/

void ecs_loc_comm__ecrit_ascii
(
 const bft_file_t  *const fic,
 const void        *const tab_val,
 const ecs_int_t          tab_nbr,
 const ecs_type_t         typ_val
) ;


/*============================================================================
 *                             Fonctions publiques
 *============================================================================*/

/*----------------------------------------------------------------------------
 *  Fonction qui initialise un communicateur
 *----------------------------------------------------------------------------*/

ecs_comm_t * ecs_comm__initialise
(
 const ecs_int_t        num_noeud ,         /* Numro du noeud associ (ou 0) */
       ecs_comm_typ_t   type_comm ,         /* Type de communicateur          */
 const ecs_int_t        echo_comm           /* Echo sur sortie principale     */
)
{

  char        * nom_fic_comm = NULL ;
  ecs_comm_t  * comm ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  BFT_MALLOC(comm, 1, ecs_comm_t) ;

  /* Construction du nom du communicateur */
  /*--------------------------------------*/

  BFT_MALLOC(comm->nom,
             strlen("enveloppe_vers_solveur") + 1,
             char) ;

  sprintf(comm->nom,
          "enveloppe_vers_solveur") ;

  /* Initialisation des autres champs */

  comm->type = type_comm ;

  comm->fic        = NULL ;

  comm->echo = echo_comm ;

  comm->num_noeud = num_noeud ;


  /* Saut d'une ligne dans le listing */

  bft_printf("\n") ;


  /* Construction des interfaces */

  BFT_MALLOC(nom_fic_comm,
             strlen("enveloppe_vers_solveur") + 1
             + (num_noeud == 0 ? 0 : 4 + 2),
             char) ;


  /* Construction du nom du fichier d'interface */
  /*--------------------------------------------*/

  if (num_noeud == 0)
    sprintf(nom_fic_comm,
            "enveloppe_vers_solveur") ;

  else
    sprintf(nom_fic_comm,
            "enveloppe_vers_solveur_n%04d",
            num_noeud) ;


  /* Info sur la creation de l'interface */

  if (num_noeud == 0)
    bft_printf(_("  Opening communication: %s\n"),
               comm->nom) ;
  else
    bft_printf(_("  Opening communication: %s (node %4d)\n"),
               comm->nom, (int)num_noeud) ;

  bft_printf_flush();


    /* Creation du descripteur de fichier d'interface */
    /*------------------------------------------------*/


  if (   comm->type == ECS_COMM_TYP_ASCII
      || comm->type == ECS_COMM_TYP_BINAIRE_FORTRAN)

    ecs_loc_comm__fic_ouvre(comm, nom_fic_comm) ;


  /* Fin */

  if (nom_fic_comm != NULL)
    BFT_FREE(nom_fic_comm) ;


  return comm ;

}


/*----------------------------------------------------------------------------
 *  Fonction qui termine la communication
 *----------------------------------------------------------------------------*/

ecs_comm_t * ecs_comm__termine
(
 ecs_comm_t *           comm
)
{

  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  if (comm->echo >= 0)
    bft_printf("\n") ;


  if (comm->num_noeud == 0)
    bft_printf(_("  Closing communication: %s\n"),
               comm->nom) ;

  else
    bft_printf(_("  Closing communication: %s (node %4d)\n"),
               comm->nom, (int)comm->num_noeud) ;


  if (comm->fic != NULL)
    comm->fic = bft_file_free(comm->fic) ;

  BFT_FREE(comm->nom) ;

  BFT_FREE(comm) ;


  return NULL ;

}


/*----------------------------------------------------------------------------
 *  Fonction d'interrogation du nom d'un communicateur
 *
 *  Cette fonction renvoie un pointeur sur un nom existant, et ne cre donc
 *   pas une nouvelle chane de caracteres. On ne doit donc pas dsallouer
 *   la chane pointe par cette fonction.
 *----------------------------------------------------------------------------*/

char * ecs_comm__ret_nom
(
 const ecs_comm_t *const comm
)
{
  return comm->nom ;
}


/*----------------------------------------------------------------------------
 * Fonction qui crit une rubrique dans le fichier d'interface
 *
 * Descriptif d'une rubrique du fichier d'interface avec le Noyau :
 *
 * 1. numro  de la rubrique (toujours 0 pour l'Enveloppe)
 * 2. nom     de la rubrique
 * 3. nombre          d'lements
 * 4. nom     du type d'lements
 * 5. valeurs des       lements
 *----------------------------------------------------------------------------*/

void ecs_comm__ecrit_rubrique
(
 const char        *const nom_rub     ,
 const ecs_int_t          nbr_elt_rub ,
       void        *const elt_rub     ,
       ecs_type_t         typ_elt_e   ,
 const ecs_comm_t  *const comm
)
{

  char nom_rub_ecr    [ECS_COMM_LNG_NOM_RUB + 1] ;

  char nom_typ_elt    [ECS_COMM_LNG_NOM_TYP_ELT + 1] ;
  char nom_typ_elt_ecr[ECS_COMM_LNG_NOM_TYP_ELT + 1] ;

  ecs_int_t num_rub ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(comm != NULL) ;
  assert(nbr_elt_rub >= 0) ;


  /* numro de la rubrique : toujours 0 pour l'Enveloppe*/

  num_rub = 0 ;


  /* nom de la rubrique */

  sprintf(nom_rub_ecr,
          "%-*.*s",
          ECS_COMM_LNG_NOM_RUB,
          ECS_COMM_LNG_NOM_RUB,
          nom_rub) ;


  /* nom du type d'elements */

  if (nbr_elt_rub != 0) {

    switch(typ_elt_e) {

    case ECS_TYPE_ecs_int_t:

      strcpy(nom_typ_elt, ECS_COMM_NOM_TYP_ELT_ENTIER) ;
      break ;

    case ECS_TYPE_ecs_real_t:

      switch(sizeof(ecs_real_t)) {
      case ECS_LOC_COMM_REEL_4:
        strcpy(nom_typ_elt, ECS_COMM_NOM_TYP_ELT_REEL_4) ;
        break ;
      case ECS_LOC_COMM_REEL_8:
        strcpy(nom_typ_elt, ECS_COMM_NOM_TYP_ELT_REEL_8) ;
        break ;
      default:
        assert(sizeof(ecs_real_t) == ECS_LOC_COMM_REEL_4 ||
               sizeof(ecs_real_t) == ECS_LOC_COMM_REEL_8   ) ;
      }
      break ;

    case ECS_TYPE_char:

      strcpy(nom_typ_elt, ECS_COMM_NOM_TYP_ELT_CHAINE) ;
      break ;

    default:

      assert(typ_elt_e == ECS_TYPE_ecs_int_t ||
             typ_elt_e == ECS_TYPE_ecs_real_t    ||
             typ_elt_e == ECS_TYPE_char       ) ;


    } /* Fin `switch (typ_elt_e)' */


    sprintf(nom_typ_elt_ecr,
            "%-*.*s",
            ECS_COMM_LNG_NOM_TYP_ELT,
            ECS_COMM_LNG_NOM_TYP_ELT,
            nom_typ_elt) ;

  }


  if (comm->echo  >= 0)

    ecs_loc_comm__echo_pre(comm) ;


  /*------------------------------------------------------------------------*/
  /* Cas d'un fichier                                                       */
  /*------------------------------------------------------------------------*/

  if (   comm->type == ECS_COMM_TYP_ASCII
      || comm->type == ECS_COMM_TYP_BINAIRE_FORTRAN) {

    /* 1. numro  de type de la rubrique (toujours 0 ici) */
    /*----------------------------------------------------*/

    ecs_loc_comm__ecrit_rec(comm,
                            (const ecs_byte_t *)(&num_rub),
                            1,
                            ECS_TYPE_ecs_int_t) ;

    /* 2. nom     de type de la rubrique */
    /*-----------------------------------*/

    ecs_loc_comm__ecrit_rec(comm,
                            (const ecs_byte_t *)nom_rub_ecr,
                            ECS_COMM_LNG_NOM_RUB,
                            ECS_TYPE_char) ;

    /* 3. nombre          d'lments */
    /*-------------------------------*/

    ecs_loc_comm__ecrit_rec(comm,
                            (const ecs_byte_t *)(&nbr_elt_rub),
                            1,
                            ECS_TYPE_ecs_int_t) ;


    if (nbr_elt_rub != 0) {

      /* 4. nom     du type d'lments */
      /*-------------------------------*/

      ecs_loc_comm__ecrit_rec(comm,
                              (const ecs_byte_t *)nom_typ_elt_ecr,
                              ECS_COMM_LNG_NOM_TYP_ELT,
                              ECS_TYPE_char) ;

      /* 5. valeurs des       lments */
      /*-------------------------------*/

      ecs_loc_comm__ecrit_rec(comm,
                              (const ecs_byte_t *)elt_rub,
                              (size_t)nbr_elt_rub,
                              typ_elt_e) ;

    } /* Fin : s'il y a des lments  crire */

  }


  /*------------------------------------------------------------------------*/
  /* Affichage eventuel                                                     */
  /*------------------------------------------------------------------------*/

  if (comm->echo >= 0)

    ecs_loc_comm__echo_entete(num_rub,
                              nom_rub_ecr,
                              nbr_elt_rub,
                              nom_typ_elt_ecr) ;

  if (comm->echo > 0)

    ecs_loc_comm__echo_donnees(comm,
                               nbr_elt_rub,
                               typ_elt_e,
                               elt_rub) ;

}


/*============================================================================
 *                              Fonctions prives
 *============================================================================*/

/*----------------------------------------------------------------------------
 *  Fonction qui construit le descripteur du fichier d'interface et initialise
 *   ce fichier par l'envoi ou la lecture d'une eventuelle "chaine magique"
 *   servant a verifier le bon format des fichiers
 *----------------------------------------------------------------------------*/

static void ecs_loc_comm__fic_ouvre
(
       ecs_comm_t        *comm          ,
 const char       *const  nom_fic_comm
)
{

  bft_file_type_t   fic_typ_comm  ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  switch(comm->type) {

  case ECS_COMM_TYP_BINAIRE_FORTRAN:

    /* Fichier d'interface au format BINAIRE FORTRAN */
    /*-----------------------------------------------*/

    fic_typ_comm = BFT_FILE_TYPE_FORTRAN_BINARY ;

    break ;

  case ECS_COMM_TYP_ASCII:

    /* Fichier d'interface au format ASCII */
    /*-------------------------------------*/

    fic_typ_comm = BFT_FILE_TYPE_TEXT ;

    break ;

  default:

    assert(comm->type == ECS_COMM_TYP_BINAIRE_FORTRAN ||
           comm->type == ECS_COMM_TYP_ASCII             ) ;

  }


  /* Creation du descripteur du fichier d'interface */
  /*------------------------------------------------*/

  comm->fic = bft_file_open(nom_fic_comm,
                            BFT_FILE_MODE_WRITE,
                            fic_typ_comm) ;

  bft_file_set_big_endian(comm->fic) ;


  /* criture d'une "chane magique" */
  /*---------------------------------*/

  ecs_loc_comm__ecrit_rec(comm,
                          "ESC_0.1",
                          7, /* longueur de la chane */
                          ECS_TYPE_char) ;

}


/*----------------------------------------------------------------------------
 *  Affichage de l'attente d'echange  d'un message
 *----------------------------------------------------------------------------*/

static void ecs_loc_comm__echo_pre
(
 const ecs_comm_t      *const comm
)
{

  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(comm != NULL) ;


  if (comm->num_noeud == 0)
    bft_printf(_("\nMessage sent on \"%s\" :\n"), comm->nom) ;
  else
    bft_printf(_("\nMessage sent on \"%s\" (node %d) :\n"), comm->nom,
               comm->num_noeud) ;

  bft_printf_flush() ;

}


/*----------------------------------------------------------------------------
 *  Affichage de l'entte d'une rubrique
 *
 *  1. numro  de la rubrique
 *  2. nom     de la rubrique
 *  3. nombre          d'lments
 *  4. nom     du type d'lments
 *----------------------------------------------------------------------------*/

static void ecs_loc_comm__echo_entete
(
       ecs_int_t              num_rub     ,
 const char            *const nom_rub     ,
       ecs_int_t              nbr_elt_rub ,
 const char            *const nom_typ_e
)
{

  char nom_rub_ecr[ECS_COMM_LNG_NOM_RUB + 1] ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  strncpy(nom_rub_ecr, nom_rub,   ECS_COMM_LNG_NOM_RUB) ;
  nom_rub_ecr[ECS_COMM_LNG_NOM_RUB] = '\0' ;

  bft_printf(_("    section number        : %d\n"
               "    section name          : \"%s\"\n"
               "    number of elements    : %d\n"),
             (int)num_rub, nom_rub_ecr, (int)nbr_elt_rub) ;


  if (nbr_elt_rub > 0) {

    char nom_typ_ecr[ECS_COMM_LNG_NOM_TYP_ELT + 1] ;

    strncpy(nom_typ_ecr, nom_typ_e, ECS_COMM_LNG_NOM_TYP_ELT) ;
    nom_typ_ecr[ECS_COMM_LNG_NOM_TYP_ELT] = '\0' ;

    bft_printf(_("    element type name     : \"%s\"\n"), nom_typ_ecr) ;

  }


  bft_printf_flush() ;

}


/*----------------------------------------------------------------------------
 *  Affichage (partiel) du contenu d'une rubrique
 *----------------------------------------------------------------------------*/

static void ecs_loc_comm__echo_donnees
(
 const ecs_comm_t    *const comm        ,
       ecs_int_t            nbr_elt_rub ,
 ecs_type_t           const typ_elt_e   ,
       void          *const elt_rub
)
{

  ecs_int_t        echo_deb ;
  ecs_int_t        echo_fin ;
  ecs_int_t        ind      ;

  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  if (nbr_elt_rub == 0) return ;

  assert(comm != NULL) ;

  echo_deb = 0 ;

  if (comm->echo * 2 < nbr_elt_rub) {

    echo_fin = comm->echo ;

    bft_printf(_("    %d first and last elements:\n"), comm->echo) ;

  }
  else {

    echo_fin = nbr_elt_rub ;

    bft_printf(_("    elements:\n")) ;

  }


  do {

    switch(typ_elt_e) {

    case ECS_TYPE_ecs_int_t: {

      ecs_int_t *elt_rub_int = (ecs_int_t *) elt_rub ;

      for (ind = echo_deb ; ind < echo_fin ; ind++)
        bft_printf("    %10d : %12d\n", ind + 1, *(elt_rub_int + ind)) ;

    }
    break ;

    case ECS_TYPE_ecs_real_t: {

      ecs_real_t *elt_rub_real = (ecs_real_t *) elt_rub ;

      for (ind = echo_deb ; ind < echo_fin ; ind++)
        bft_printf("    %10d : %12.5e\n", ind + 1, *(elt_rub_real + ind)) ;

    }
    break ;

    case ECS_TYPE_char: {

      char *elt_rub_char = (char *) elt_rub ;

      for (ind = echo_deb ; ind < echo_fin ; ind++) {
        if (*(elt_rub_char + ind) != '\0')
          bft_printf("    %10d : '%c'\n", ind + 1, *(elt_rub_char + ind)) ;
        else
          bft_printf("    %10d : '\\0'\n", ind + 1) ;
      }

    }
    break ;

    default:

      assert(typ_elt_e == ECS_TYPE_ecs_int_t  ||
             typ_elt_e == ECS_TYPE_ecs_real_t ||
             typ_elt_e == ECS_TYPE_char         ) ;

    }


    if (echo_fin < nbr_elt_rub) {

      bft_printf(_("    ..........   ............\n")) ;

      echo_deb = nbr_elt_rub - comm->echo ;
      echo_fin = nbr_elt_rub ;

    }
    else {

      assert(echo_fin == nbr_elt_rub) ;

      echo_fin = nbr_elt_rub + 1 ;

    }


  } while (echo_fin <= nbr_elt_rub) ;


  bft_printf_flush() ;

}


/*----------------------------------------------------------------------------
 *  Fonction qui crit un enregistrement dans le fichier d'interface
 *
 *  Cette fonction slectionne le mode d'criture de l'enregistrement
 *  (ASCII ou BINAIRE FORTRAN) suivant le type du fichier
 *  (ASCII ou BINAIRE FORTRAN)
 *----------------------------------------------------------------------------*/

static void ecs_loc_comm__ecrit_rec
(
 const ecs_comm_t  *const comm  ,
 const ecs_byte_t  *      rec   ,
 const size_t             nbr   ,
       ecs_type_t         typ_e
)
{

  size_t taille ;

  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/

  assert(comm != NULL) ;
  assert(rec  != NULL) ;


  switch(bft_file_get_type(comm->fic)) {

  case BFT_FILE_TYPE_TEXT:

    /* criture de l'enregistrement dans le fichier ASCII */
    /*----------------------------------------------------*/

    ecs_loc_comm__ecrit_ascii(comm->fic,
                              (const void *)rec,
                              (ecs_int_t)nbr,
                              typ_e) ;

    break ;


  case BFT_FILE_TYPE_FORTRAN_BINARY:

    /* Dtermination de la taille associe au type  envoyer */

    switch (typ_e) {
    case ECS_TYPE_ecs_int_t:
      taille = sizeof(ecs_int_t) ;
      break ;
    case ECS_TYPE_ecs_real_t:
      taille = sizeof(ecs_real_t) ;
      break ;
    case ECS_TYPE_char:
      taille = sizeof(char) ;
      break ;
    default:
      assert(typ_e == ECS_TYPE_ecs_int_t  ||
             typ_e == ECS_TYPE_ecs_real_t ||
             typ_e == ECS_TYPE_char ) ;
    }

    /* criture de l'enregistrement dans le fichier */
    /*----------------------------------------------*/

    bft_file_write(rec,
                   taille,
                   nbr,
                   comm->fic) ;

    break ;

  default:

    assert (   (   bft_file_get_type(comm->fic)
                == BFT_FILE_TYPE_TEXT)
            || (   bft_file_get_type(comm->fic)
                == BFT_FILE_TYPE_FORTRAN_BINARY)) ;

  }


}


/*----------------------------------------------------------------------------
 *  Fonction d'impression d'un tableau en ASCII
 *
 *  Si les valeurs sont de type "caractre", les valeurs sont crites
 *   sur une seule ligne
 *----------------------------------------------------------------------------*/

void ecs_loc_comm__ecrit_ascii
(
 const bft_file_t  *const fic,
 const void        *const tab_val,
 const ecs_int_t          tab_nbr,
 const ecs_type_t         typ_val
)
{
  ecs_int_t  nbr_ligne ;
  ecs_int_t  reste_col ;
  ecs_int_t  cpt_val ;
  ecs_int_t  icol ;
  ecs_int_t  ilig ;


  /* Instructions */

  assert (tab_val != NULL) ;

  cpt_val = 0 ;

  nbr_ligne = tab_nbr / ECS_COMM_FMT_ASCII_NBR_COLONNE ;  /* Division entire */
  reste_col = tab_nbr - nbr_ligne * ECS_COMM_FMT_ASCII_NBR_COLONNE ;


  /* Impression des valeurs */
  /*========================*/

  switch (typ_val) {

  case ECS_TYPE_char:

    {
      const char    *const tab_val_char = (const char *)tab_val ;

      cpt_val = 0 ;
      while (cpt_val < tab_nbr) {
        bft_file_printf(fic, "%s\n", tab_val_char + cpt_val) ;
        cpt_val += strlen(tab_val_char + cpt_val) + 1 ;
      }

    }

    break;

  case ECS_TYPE_ecs_int_t:

    {
      const ecs_int_t  *const tab_val_int = (const ecs_int_t *) tab_val ;

      for (ilig = 0 ; ilig < nbr_ligne ; ilig++) {

        for (icol = 0 ;
             icol < ECS_COMM_FMT_ASCII_NBR_COLONNE ;
             icol++ , cpt_val++)
          bft_file_printf(fic, ECS_COMM_FMT_ASCII_INT,
                          tab_val_int[cpt_val]) ;
        bft_file_printf(fic, "\n") ;

      }

      if (reste_col != 0) {

        for (icol = 0 ;
             icol < reste_col ;
             icol++ , cpt_val++)
          bft_file_printf(fic, ECS_COMM_FMT_ASCII_INT,
                          tab_val_int[cpt_val]) ;
        bft_file_printf(fic, "\n") ;

      }

    }

    break ;

  case ECS_TYPE_ecs_real_t:

    {
      const ecs_real_t  *const tab_val_real = (const ecs_real_t *)tab_val ;

      for (ilig = 0 ; ilig < nbr_ligne ; ilig++) {

        for (icol = 0 ;
             icol < ECS_COMM_FMT_ASCII_NBR_COLONNE ;
             icol++ , cpt_val++)
          bft_file_printf(fic, ECS_COMM_FMT_ASCII_REAL,
                          tab_val_real[cpt_val]) ;
        bft_file_printf(fic, "\n") ;

      }

      if (reste_col != 0) {

        for (icol = 0 ;
             icol < reste_col ;
             icol++ , cpt_val++)
          bft_file_printf(fic, ECS_COMM_FMT_ASCII_REAL,
                          tab_val_real[cpt_val]) ;
        bft_file_printf(fic, "\n") ;

      }

    }

    break ;

  default:

    assert(   typ_val != ECS_TYPE_ecs_int_t
           || typ_val != ECS_TYPE_ecs_real_t
           || typ_val != ECS_TYPE_char) ;

  } /* Fin : switch (typ_val_e) */

}

