/*============================================================================
 * Dfinitions, variables globales, et fonctions de base
 *============================================================================*/

/*
                      Code_Saturne version 1.3
                      ------------------------

  This file is part of the Code_Saturne Preprocessor, element of the
  Code_Saturne CFD tool.

  Copyright (C) 1999-2007 EDF S.A., France

  contact: saturne-support@edf.fr

  The Code_Saturne Preprocessor is free software; you can redistribute it
  and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  The Code_Saturne Preprocessor is distributed in the hope that it will be
  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with the Code_Saturne Preprocessor; if not, write to the
  Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor,
  Boston, MA  02110-1301  USA
*/

#include "ecs_config.h"


/* includes systme ou BFT */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#if defined(HAVE_CGNS)
#include <cgnslib.h>
#endif

#include <bft_error.h>
#include <bft_printf.h>


/* Includes librairie */

#include "ecs_def.h"


/*=============================================================================
 * Dfinitions de variables globales
 *============================================================================*/

/* Date de compilation */

char      ecs_glob_build_date[]   = __DATE__;

/* Support de la librairie d'change de maillages et champs CGNS */

#if defined(HAVE_CGNS)
int       ecs_glob_have_cgns      = 1;
#if defined(CGNS_VERSION)
int       ecs_glob_cgns_ver_maj  = CGNS_VERSION/1000;
int       ecs_glob_cgns_ver_min  = (CGNS_VERSION % 1000) / 100;
int       ecs_glob_cgns_ver_rel  = (CGNS_VERSION % 100) / 10;
#elif defined(NofValidAreaTypes)
int       ecs_glob_cgns_ver_maj  = 2;
int       ecs_glob_cgns_ver_min  = 2;
int       ecs_glob_cgns_ver_rel  = -1;
#else
int       ecs_glob_cgns_ver_maj  = 2;
int       ecs_glob_cgns_ver_min  = 1;
int       ecs_glob_cgns_ver_rel  = -1;
#endif
#else
int       ecs_glob_have_cgns     = 0;
int       ecs_glob_cgns_ver_maj  = 0;
int       ecs_glob_cgns_ver_min  = 0;
int       ecs_glob_cgns_ver_rel  = 0;
#endif

/* Support de la librairie d'change de maillages et champs MED */

#if defined(HAVE_MED)
int       ecs_glob_have_med     = 1;
#else
int       ecs_glob_have_med     = 0;
#endif

/* Support du dcoupeur de graphes de connectivit METIS */

#if defined(HAVE_METIS)
int       ecs_glob_have_metis     = 1;
#else
int       ecs_glob_have_metis     = 0;
#endif

/* Dfinitions locales */


/*============================================================================
 * Prototypes de fonctions prives
 *============================================================================*/

static void ecs_loc_def__error_handler
(
 const char     *const nom_fic,
 const int             num_ligne,
 const int             code_err_sys,
 const char     *const format,
 const va_list         arg_ptr
);


/*----------------------------------------------------------------------------
 *  Fonction qui compte la largeur d'une chane de caractres, en tenant
 *  compte de la possibilit que cette chane soit de type UTF-8.
 *----------------------------------------------------------------------------*/

int  ecs_loc_def__strlen
(
 const char  * chaine
);


/*============================================================================
 * Fonctions publiques
 *============================================================================*/

/*----------------------------------------------------------------------------
 * Fonction d'initialisation de la gestion des erreurs
 *----------------------------------------------------------------------------*/

void ecs_init_gestion_erreur
(
  void
)
{
  bft_error_handler_set(ecs_loc_def__error_handler);
}


/*----------------------------------------------------------------------------
 * Fonction d'arrt
 *----------------------------------------------------------------------------*/

void ecs_exit
(
  const ecs_int_t  statut
)
{
  if (statut == EXIT_FAILURE) {

    fprintf (stdout, "\n\n %s\n\n", _("Abnormal end"));

#if defined(DEBUG) || !defined(NDEBUG)
    assert(0);
#endif

  }

  exit (statut);

}


/*----------------------------------------------------------------------------
 * Fonction d'impression d'un avertissement
 *----------------------------------------------------------------------------*/

void ecs_warn
(
  void
)
{
  bft_printf(_("\n"
               "Warning\n"
               "=======\n"));
}


/*----------------------------------------------------------------------------
 *  Fonction qui imprime une chane de caractres avec une largeur
 *  de colonne donne.
 *----------------------------------------------------------------------------*/

void ecs_print_padded_str
(
 const char  *str,
       int    width
)
{

  int lng = ecs_loc_def__strlen(str);

  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/

  if (str != NULL)
    bft_printf(str);

  if (width > lng)
    bft_printf("%-*s", width-lng, "" );

}


/*============================================================================
 * Fonctions prives
 *============================================================================*/

static void ecs_loc_def__error_handler
(
 const char     *const nom_fic,
 const int             num_ligne,
 const int             code_err_sys,
 const char     *const format,
 const va_list         arg_ptr
)
{
  bft_printf_flush();

  fprintf(stderr, _("\n"
                    "Error in ECS execution\n"
                    "======================\n"));

  if (code_err_sys != 0)
    fprintf(stderr, _("\nSystem error: %s\n"), strerror(code_err_sys));

  fprintf(stderr, _("\n%s:%d: Fatal error.\n\n"), nom_fic, num_ligne);

  vfprintf(stderr, format, arg_ptr);

  fprintf(stderr, "\n\n");

  assert(0);   /* Utilisation de assert pour interception sous un debugger */

  exit(EXIT_FAILURE);
}


/*----------------------------------------------------------------------------
 *  Fonction qui compte la largeur d'une chane de caractres, en tenant
 *  compte de la possibilit que cette chane soit de type UTF-8.
 *----------------------------------------------------------------------------*/

int  ecs_loc_def__strlen
(
 const char  * chaine
)
{
  static int mode_utf8 = -1;

  int lng = 0;
  int retval = 0;

  if (mode_utf8 == -1) {
    char *lang = getenv("LANG");
    mode_utf8 = 0;
    if (lang != NULL) {
      if (strcmp(lang + strlen(lang) - 5, "UTF-8") == 0)
        mode_utf8 = 1;
    }
  }

  if (chaine != NULL) {

    lng = strlen(chaine);

    if (mode_utf8 == 0)
      retval = lng;

    else if (mode_utf8 == 1) {

      int ind;
      ecs_bool_t multibyte = ECS_FALSE;

      for (ind = 0; ind < lng; ind++) {

        char c = chaine[ind];

        if (multibyte == ECS_FALSE || (c < 0x80 || c > 0xBF)) {

          multibyte = ECS_FALSE;

          if (c <= 0x7F) {
            retval++;
          }
          else if (c >= 0xC0 || c <= 0xFD) {
            multibyte = ECS_TRUE;
            retval++;
          }

        }

      }

    }

  }

  return retval;

}

